﻿using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using OAuth_Login.Global;
using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.IO;
using System.Net;
using System.Net.Http;
using System.Text;
using System.Web;
using System.Windows.Forms;

namespace OAuth_Login.OAuth
{


	public class COAuth_Naver : COAuth
	{
		/// <summary>
		/// 사용할 브라우저 컨트롤
		/// </summary>
		private WebBrowser m_webAuth;

		public COAuth_Naver(WebBrowser wbAuth)
		{
			this.m_webAuth = wbAuth;

			this.Reset();
		}

		protected override void Reset()
		{
			base.ClientID = [1번에서 등록할때 받은 클라이언트 아이디];
			base.ClientSecret = [1번에서 등록할때 받은 클라이언트 시크릿];
			base.RedirectURL = "http://blog.danggun.net/";

			base.RedirectURL_uri = new Uri(this.RedirectURL);
			

			//확인용 키 생성
			base.State = (new Random()).Next().ToString();
			base.AuthToken = "";


			//인증요청용 전체 URL(쿼리 포함) 생성
			NameValueCollection listAuthURL_QueryString
				= HttpUtility.ParseQueryString(string.Empty);
			listAuthURL_QueryString["response_type"] = "code";
			listAuthURL_QueryString["client_id"] = base.ClientID;
			listAuthURL_QueryString["redirect_uri"] = base.RedirectURL;
			listAuthURL_QueryString["state"] = base.State;

			base.AuthURL = "https://nid.naver.com/oauth2.0/authorize?"
				+ listAuthURL_QueryString.ToString();


			//이벤트 연결
			this.m_webAuth.DocumentCompleted -= wbAuth_DocumentCompleted;
			this.m_webAuth.DocumentCompleted += wbAuth_DocumentCompleted;
		}

		private void wbAuth_DocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
		{

			//키가 넘어오는 페이지인지 확인한다.
			if ((base.RedirectURL_uri.Host == this.m_webAuth.Url.Host)
				&& (base.RedirectURL_uri.LocalPath == this.m_webAuth.Url.LocalPath))
			{//인증키가 넘어온 페이지
				//쿼리갯수를 확인한다.
				NameValueCollection query 
					= HttpUtility.ParseQueryString(this.m_webAuth.Url.Query);
				if (2 == query.Count)
				{//인증용 쿼리다.
					if( query["state"].ToString() == this.State)
					{//확인용 키가 일치 한다.
						base.AuthToken = query["code"].ToString();
						base.OnOAuth_Authentication_Result_Call(
							string.Format("code : {0} \n state : {1}"
											, base.AuthToken
											, base.State));
					}
				}
			}
		}//end wbAuth_DocumentCompleted

		/// <summary>
		/// 인증을 시작합니다.
		/// </summary>
		public override void StartAuth()
		{
			//외부로 요청 주소를 보냄
			base.OnOAuth_RequestURL_Create_Call(base.AuthURL);
			
			//인증 시작
            this.m_webAuth.Navigate(new Uri(base.AuthURL));
		}

		public void Blog_List_Get()
		{
			WebClient webClient;
			Stream stream;
			JToken token;


			//1. 로그인된 코드를 이용하여 엑세스 토큰을 받아옵니다.			
			//https://nid.naver.com/devcenter/docs.nhn?menu=API (2. 접근 토큰 발급 요청)

			string sAccessToken_Url = "https://nid.naver.com/oauth2.0/token";

			//쿼리 세팅
			webClient = new WebClient();
			webClient.QueryString.Add("grant_type", "authorization_code");
			webClient.QueryString.Add("client_id", this.ClientID);
			webClient.QueryString.Add("client_secret", this.ClientSecret);
			webClient.QueryString.Add("code", this.AuthToken);
			webClient.QueryString.Add("state", this.State);

			//요청
			stream = webClient.OpenRead(sAccessToken_Url);
			//결과 받기
            token = JObject.Parse(new StreamReader(stream).ReadToEnd());
			//Json 파싱
			string access_token = token.SelectToken("access_token").ToString();
			string refresh_token = token.SelectToken("refresh_token").ToString();
			string token_type = token.SelectToken("token_type").ToString();
			string expires_in = token.SelectToken("expires_in").ToString();



			//2. 네이버 블로그 카테고리 조회
			//http://developer.naver.com/wiki/pages/BlogAPISpec (2. 블로그 카테고리 조회 API)

			string sListCategoryApiUri = "https://openapi.naver.com/blog/listCategory.json";
			string sBlogID = "c1dong8";

			webClient = new WebClient();
			webClient.Headers.Add("Authorization", "Bearer " + access_token);
			webClient.QueryString.Add("blogId", sBlogID);

			//요청
			stream = webClient.OpenRead(sListCategoryApiUri);
			//결과 받기
			string responseJSON2 = new StreamReader(stream).ReadToEnd();

			//결과 출력
			base.OndgOAuth_Output_Call(responseJSON2);

		}

	}
}
